# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from transaction import commit

import unittest


class LuogoBehaviorIndexerFunctionalTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_FUNCTIONAL_TESTING
    maxDiff = None

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.catalog = api.portal.get_tool("portal_catalog")
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        fti = DexterityFTI("venueitem")
        self.portal.portal_types._setObject("venueitem", fti)
        fti.klass = "plone.dexterity.content.Item"
        fti.behaviors = (
            "plone.app.content.interfaces.INameFromTitle",
            "plone.app.dexterity.behaviors.metadata.IBasic",
            "plone.app.dexterity.behaviors.metadata.ICategorization",
            "collective.geolocationbehavior.geolocation.IGeolocatable",
            "design.plone.contenttypes.behavior.additional_help_infos",
            "design.plone.contenttypes.behavior.argomenti",
            "plone.leadimage",
            "volto.preview_image",
            "collective.address.behaviors.IAddress",
            "design.plone.contenttypes.behavior.luogo",
            "plone.textindexer",
        )
        self.fti = fti

        self.portal.invokeFactory("venueitem", id="venue", title="venue")
        self.venue = self.portal.venue
        commit()

    def test_luogo_behavior_fields_inexed_for_venue(self):
        # Non sembra deterministico il testing delle cose indicizzate con
        # plone.textindexer. Per ora togliamo. Poi se capiamo
        # come gestire lo rimetteremo.
        return
        self.assertTrue(True)
        res = api.content.find(UID=self.venue.UID())
        rid = res[0].getRID()

        self.assertEqual(
            self.catalog.getIndexDataForRID(rid)["SearchableText"], ["venue"]
        )

        self.venue.quartiere = "quartiere"
        self.venue.circoscrizione = "Nord/ovest"
        self.venue.descrizione_breve = {
            "blocks": {"123456": {"searchableText": "breve"}}
        }
        self.venue.orario_pubblico = {
            "blocks": {"123456": {"searchableText": "orario"}}
        }
        self.venue.reindexObject(idxs="SearchableText")
        commit()

        index_data = self.catalog.getIndexDataForRID(rid)
        self.assertIn("quartiere", index_data["SearchableText"])
        self.assertIn("nord", index_data["SearchableText"])
        self.assertIn("ovest", index_data["SearchableText"])
        self.assertIn("breve", index_data["SearchableText"])
        self.assertIn("orario", index_data["SearchableText"])
        self.assertEqual(
            index_data["SearchableText"],
            ["venue", "quartiere", "nord", "ovest", "breve", "orario"],
        )

        res = api.content.find(SearchableText="breve")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0].UID, self.venue.UID())



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from redturtle.bandi.interfaces.settings import IBandoSettings

import unittest


class TestBando(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_bando_folder_deepening_addable_types(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            ("File", "Link", "Modulo"),
            portal_types["Bando Folder Deepening"].allowed_content_types,
        )

    def test_bando_view_base(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(portal_types["Bando"].default_view, "view")
        self.assertEqual(portal_types["Bando"].view_methods, ("view",))

    def test_disabled_default_ente(self):
        default_ente = api.portal.get_registry_record(
            "default_ente", interface=IBandoSettings
        )
        self.assertEqual(default_ente, ())

    def test_bando_substructure_created(self):
        bando = api.content.create(container=self.portal, type="Bando", title="Bando")

        self.assertIn("documenti", bando.keys())
        self.assertIn("comunicazioni", bando.keys())
        self.assertIn("esiti", bando.keys())



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


# TODO: merge with NEWS
@provider(IFormFieldProvider)
class IDatasetCorrelati(model.Schema):
    dataset_correlati = RelationList(
        title=_("dataset_correlati_label", default="Dataset correlati"),
        description=_(
            "dataset_correlati_help",
            default="Seleziona una lista di schede dataset collegate a questo"
            " contenuto.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )
    form.widget(
        "dataset_correlati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Dataset"],
            "maximumSelectionSize": 50,
        },
    )

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["dataset_correlati"],
    )


@implementer(IDatasetCorrelati)
@adapter(IDexterityContent)
class DatasetCorrelati(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.messaggio import IMessaggio
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IMessaggio)
class Messaggio(Container):
    """Marker interface for Messaggio"""



# -*- coding: utf-8 -*-
from plone import api


def bandoCreateHandler(bando, event):
    """ """
    folders = [
        {"id": "documenti", "title": "Documenti"},
        {"id": "comunicazioni", "title": "Comunicazioni"},
        {"id": "esiti", "title": "Esiti"},
    ]
    for mapping in folders:
        if mapping["id"] not in bando:
            api.content.create(
                type="Bando Folder Deepening",
                title=mapping["title"],
                id=mapping["id"],
                container=bando,
            )



# -*- coding: utf-8 -*-
from DateTime import DateTime
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.utils import getToolByName
from transaction import commit
from uuid import uuid4
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest


class TestLuogo(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING
    maxDiff = None

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]

    def test_behaviors_enabled_for_luogo(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Venue"].behaviors,
            (
                "plone.app.content.interfaces.INameFromTitle",
                "plone.app.dexterity.behaviors.id.IShortName",
                "plone.app.dexterity.behaviors.metadata.IBasic",
                "plone.app.dexterity.behaviors.metadata.ICategorization",
                "plone.excludefromnavigation",
                "plone.relateditems",
                "plone.leadimage",
                "volto.preview_image",
                "design.plone.contenttypes.behavior.contatti_venue",
                "design.plone.contenttypes.behavior.luogo",
                "design.plone.contenttypes.behavior.argomenti",
                "design.plone.contenttypes.behavior.address_venue",
                "design.plone.contenttypes.behavior.geolocation_venue",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "design.plone.contenttypes.behavior.luoghi_correlati",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "collective.taxonomy.generated.tipologia_luogo",
            ),
        )

    def test_luogo_ct_title(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual("Luogo", portal_types["Venue"].title)


class TestLuogoApi(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.news = api.content.create(
            container=self.portal, type="News Item", title="TestNews"
        )
        self.news_published = api.content.create(
            container=self.portal, type="News Item", title="TestNews published"
        )
        self.service = api.content.create(
            container=self.portal, type="Servizio", title="TestService"
        )
        self.uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="TestUO"
        )

        self.venue = api.content.create(
            container=self.portal, type="Venue", title="TestVenue"
        )

        api.content.transition(obj=self.news_published, transition="publish")
        self.news_published.setEffectiveDate(DateTime())
        self.news_published.reindexObject()

        intids = getUtility(IIntIds)

        venue = RelationValue(intids.getId(self.venue))
        self.news.luoghi_correlati = [venue]
        self.news_published.luoghi_correlati = [venue]
        self.service.dove_rivolgersi = [venue]
        self.uo.luoghi_correlati = [venue]
        pcatalog = getToolByName(self.portal, "portal_catalog")
        pcatalog.manage_reindexIndex(ids=["service_venue", "uo_location", "news_venue"])
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_venus_addable_types(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            ("Folder", "Link", "Image", "File"),
            portal_types["Venue"].allowed_content_types,
        )

    def test_venue_geolocation_deserializer_wrong_structure(self):
        venue = api.content.create(
            container=self.portal, type="Venue", title="Example venue"
        )

        commit()
        self.assertEqual(venue.geolocation, None)

        response = self.api_session.patch(
            venue.absolute_url(),
            json={"@type": "Venue", "title": "Foo", "geolocation": {"foo": "bar"}},
        )
        # message = response.json()["message"]

        self.assertEqual(400, response.status_code)
        # TODO: anzichè `invalid geolocation data` ritorna
        #       `Il campo geolocation è obbligatorio`
        # self.assertIn("Invalid geolocation data", message)
        # TODO: i dati vanno verificati con una chiamata alla api_session
        # self.assertEqual(venue.geolocation, None)

    def test_venue_geolocation_deserializer_right_structure(self):
        venue = api.content.create(
            container=self.portal, type="Venue", title="Example venue"
        )

        commit()
        self.assertEqual(venue.geolocation, None)

        response = self.api_session.patch(
            venue.absolute_url(),
            json={
                "@type": "Venue",
                "title": "Foo",
                "geolocation": {"latitude": 11.0, "longitude": 10.0},
            },
        )

        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json()["message"],
            "[{'error': 'ValidationError', "
            "'message': 'Il campo modalita_accesso è obbligatorio'}]",
        )

        text_uuid = str(uuid4())
        response = self.api_session.patch(
            venue.absolute_url(),
            json={
                "@type": "Venue",
                "title": "Foo",
                "geolocation": {"latitude": 11.0, "longitude": 10.0},
                "modalita_accesso": {
                    "blocks": {
                        text_uuid: {
                            "@type": "text",
                            "text": {"blocks": [{"text": "Test", "type": "paragraph"}]},
                        }
                    },
                    "blocks_layout": {"items": [text_uuid]},
                },
            },
        )
        self.assertEqual(response.status_code, 204)
        # TODO: i dati vanno verificati con una chiamata alla api_session
        # self.assertEqual(venue.geolocation.longitude, 10.0)
        # self.assert ... (venue.modalita_accesso)

    def test_venue_services(self):
        response = self.api_session.get(self.venue.absolute_url() + "?fullobjects")
        self.assertTrue(
            response.json()["venue_services"][0]["@id"],
            self.service.absolute_url(),
        )

    def test_venue_news(self):
        response = self.api_session.get(self.venue.absolute_url() + "?fullobjects")
        res = response.json()

        self.assertEqual(len(res["related_news"]), 2)
        self.assertEqual(
            res["related_news"][0]["@id"], self.news_published.absolute_url()
        )
        self.assertEqual(
            res["related_news"][1]["@id"],
            self.news.absolute_url(),
        )



# -*- coding: utf-8 -*-
from collective.venue.interfaces import IVenue
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces.persona import IPersona
from design.plone.contenttypes.interfaces.servizio import IServizio
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IContattiUnitaOrganizzativa(model.Schema):
    contact_info = RelationList(
        title=_(
            "contact_info_label",
            default="Punti di contatto dell'unità organizzativa",
        ),
        description=_(
            "contact_info_help",
            default="Contatti dell'unità organizzativa.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Informazioni di contatto"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )
    orario_pubblico = BlocksField(
        title=_("orario_pubblico_label", default="Orario per il pubblico"),
        description=_(
            "orario_pubblico_help",
            default="Indicare eventuali orari di accesso al pubblico",
        ),
        required=False,
    )

    form.widget(
        "contact_info",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["PuntoDiContatto"],
        },
    )
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["contact_info", "orario_pubblico"],
    )

    textindexer.searchable("orario_pubblico")


@provider(IFormFieldProvider)
class IContattiPersona(model.Schema):
    contact_info = RelationList(
        title=_(
            "contact_info_label",
            default="Punti di contatto",
        ),
        description=_(
            "contact_info_help",
            default="Punti di contatto della persona.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Punti di contatto"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    form.widget(
        "contact_info",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["PuntoDiContatto"],
        },
    )
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["contact_info"],
    )


@provider(IFormFieldProvider)
class IContattiServizio(model.Schema):
    contact_info = RelationList(
        title=_(
            "contact_info_label",
            default="Contatti",
        ),
        description=_(
            "contact_info_help",
            default="I contatti per il servizio.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Contatti"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )
    form.widget(
        "contact_info",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["PuntoDiContatto"],
        },
    )
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["contact_info"],
    )


@provider(IFormFieldProvider)
class IContattiVenue(model.Schema):
    contact_info = RelationList(
        title=_(
            "contact_info_label",
            default="Punti di contatto",
        ),
        description=_(
            "contact_info_help",
            default="Telefono, mail o altri punti di contatto.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Punti di contatto"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )
    form.widget(
        "contact_info",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["PuntoDiContatto"],
        },
    )
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["contact_info"],
    )


@provider(IFormFieldProvider)
class IContattiEvent(model.Schema):
    contact_info = RelationList(
        title=_(
            "contact_info_label",
            default="Punti di contatto",
        ),
        description=_(
            "contact_info_help",
            default="Relazione con i punti di contatto dell'evento.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Punti di contatto"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )
    form.widget(
        "contact_info",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["PuntoDiContatto"],
        },
    )
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["contact_info"],
    )


@implementer(IContattiEvent)
@adapter(IContattiEvent)
class ContattiEvent(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IContattiPersona)
@adapter(IPersona)
class ContattiPersona(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IContattiServizio)
@adapter(IServizio)
class ContattiServizio(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IContattiUnitaOrganizzativa)
@adapter(IUnitaOrganizzativa)
class ContattiUnitaOrganizzativa(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IContattiVenue)
@adapter(IVenue)
class ContattiVenue(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from plone import api
from Products.CMFPlone.interfaces import ISelectableConstrainTypes


def praticaCreateHandler(pratica, event):
    """
    Complete content type Pratica setup on added event, generating
    missing folders, fields, etc.

    @param pratica: Content item

    @param event: Event that triggers the method (onAdded event)
    """

    allegati = api.content.create(type="Folder", title="Allegati", container=pratica)

    allegatiConstraints = ISelectableConstrainTypes(allegati)
    allegatiConstraints.setConstrainTypesMode(1)
    allegatiConstraints.setLocallyAllowedTypes(("File",))



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.namedfile import field
from plone.supermodel import model
from zope import schema


class IRicevutaPagamento(model.Schema):
    """Marker interface for RicevutaPagamento"""

    numero_protocollo = schema.Id(
        title=_("numero_protocollo", default="Numero protocollo"),
        required=True,
    )

    stampa_ricevuta = field.NamedFile(
        title=_("stampa_ricevuta", default="Stampa ricevuta"), required=True
    )

    data_pagamento = schema.Date(
        title=_("data_pagamento", default="Data pagamento"), required=True
    )

    importo_pagato = schema.TextLine(
        title=_("importo_pagato", default="Importo pagato"), required=True
    )

    modalita_pagamento = schema.TextLine(
        title=_("modalita_pagamento", default="Modalità pagamento"),
        required=True,
    )

    # TODO: aggiungere tassonomia e vocabolario rilevante
    servizio_origine = schema.Choice(
        title=_("servizio_origine_ricevuta", default="Servizio che origina la pratica"),
        # vocabolario di riferimento sara' il servizio che genera il task e
        # permette di soddisfarlo
        vocabulary="design.plone.contenttypes.Mockup",
        required=False,
    )

    pratica_associata = field.NamedFile(
        title=_("pratica_associata_ricevuta", default="Pratica associata al pagamento"),
        required=True,
    )

    # TODO: capire se ci sono altri esiti oltre ai banali "accettato",
    # "rifiutato", "in attesa di conferma",
    # e costruire di conseguenza un vocabolario adeguato
    esito = schema.Choice(
        title=_("esito", default="Esito"),
        vocabulary="design.plone.contenttypes.Mockup",
        required=True,
    )

    allegato = field.NamedFile(title=_("allegato", default="Allegato"), required=False)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.schema_overrides import SchemaTweaks
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.autoform.interfaces import IFormFieldProvider
from plone.restapi.testing import RelativeSession
from zope.component import provideAdapter

import transaction
import unittest


class TestEvent(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING
    maxDiff = None

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]

    def test_behaviors_enabled_for_event(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Event"].behaviors,
            (
                "plone.eventbasic",
                "plone.leadimage",
                "volto.preview_image",
                "design.plone.contenttypes.behavior.argomenti_evento",
                "plone.eventrecurrence",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "design.plone.contenttypes.behavior.evento",
                "design.plone.contenttypes.behavior.luoghi_correlati_evento",
                "design.plone.contenttypes.behavior.address_event",
                "design.plone.contenttypes.behavior.geolocation_event",
                "design.plone.contenttypes.behavior.strutture_correlate",
                "design.plone.contenttypes.behavior.contatti_event",
                "plone.dublincore",
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.relateditems",
                "plone.versioning",
                "plone.locking",
                "plone.constraintypes",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "collective.taxonomy.generated.tipologia_evento",
            ),
        )

    def test_event_addable_types(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            sorted(("Image", "File", "Link", "Event", "Document")),
            sorted(portal_types["Event"].allowed_content_types),
        )


class TestEventApi(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.event = api.content.create(
            container=self.portal, type="Event", title="Evento"
        )
        provideAdapter(SchemaTweaks, (IFormFieldProvider,), name="schema.tweaks")
        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_event_substructure_created(self):
        event = self.portal["evento"]

        self.assertEqual(
            sorted(["documenti", "immagini", "sponsor_evento", "video"]),
            sorted(event.keys()),
        )

        self.assertEqual(event["immagini"].portal_type, "Document")
        self.assertEqual(event["immagini"].constrain_types_mode, 1)
        self.assertEqual(event["immagini"].locally_allowed_types, ("Image", "Link"))

        self.assertEqual(event["sponsor_evento"].portal_type, "Document")
        self.assertEqual(event["sponsor_evento"].constrain_types_mode, 1)
        self.assertEqual(event["sponsor_evento"].locally_allowed_types, ("Link",))

        self.assertEqual(event["documenti"].portal_type, "Document")
        self.assertEqual(event["documenti"].constrain_types_mode, 1)
        self.assertEqual(event["documenti"].locally_allowed_types, ("File",))

        self.assertEqual(event["video"].portal_type, "Document")
        self.assertEqual(event["video"].constrain_types_mode, 1)
        self.assertEqual(event["video"].locally_allowed_types, ("Link",))

        self.assertEqual(api.content.get_state(obj=event["immagini"]), "published")
        self.assertEqual(api.content.get_state(obj=event["video"]), "published")
        self.assertEqual(
            api.content.get_state(obj=event["sponsor_evento"]), "published"
        )
        self.assertEqual(api.content.get_state(obj=event["documenti"]), "published")



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class MoveNewsItemView(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        # default values are set in italian
        self.request["LANGUAGE"] = "it"
        self.view = api.content.get_view(
            "change_news_type", context=self.portal, request=self.request
        )

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.news_container = api.content.create(
            type="Folder",
            title="News container",
            container=self.portal,
        )
        self.news_item = api.content.create(
            type="News Item",
            title="news item",
            tipologia_notizia=["notizia"],
            container=self.portal,
        )
        self.news_item1 = api.content.create(
            type="News Item",
            title="news item1",
            tipologia_notizia=["notizia"],
            container=self.news_container,
        )

    def test_substitute_news_type(self):
        new_news_type = "comunicato_stampa"
        self.view.request.form["news_type_in_catalog"] = "notizia"
        self.view.request.form["news_type_portal"] = new_news_type
        self.view.request.form["substitute"] = "true"

        # mock the helper methods of our view
        self.view.news_types = lambda: [new_news_type]

        # self.portal.portal_catalog(ti)
        self.view.substitute_news_type()

        self.assertEqual(new_news_type, self.news_item.tipologia_notizia)
        self.assertEqual(new_news_type, self.news_item1.tipologia_notizia)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import RelativeSession
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest


class SummarySerializerTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.document = api.content.create(
            container=self.portal, type="Document", title="Document"
        )
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_has_children_returned_in_get_content(self):
        api.content.create(container=self.document, type="Document", title="empty")
        api.content.create(container=self.document, type="Document", title="filled")

        api.content.create(
            container=self.document["filled"], type="Document", title="Example"
        )
        commit()

        response = self.api_session.get(self.document.absolute_url())

        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 2)
        self.assertEqual(items[0]["title"], "empty")
        self.assertFalse(items[0]["has_children"])
        self.assertEqual(items[1]["title"], "filled")
        self.assertTrue(items[1]["has_children"])

    def test_has_children_not_returned_in_searches(self):
        api.content.create(container=self.document, type="Document", title="empty")
        api.content.create(container=self.document, type="Document", title="filled")

        api.content.create(
            container=self.document["filled"], type="Document", title="Example"
        )
        commit()

        response = self.api_session.get(
            "/@search?portal_type=Document&path.query=/plone/document&path.depth=1"
        )

        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 2)
        self.assertEqual(items[0]["title"], "empty")
        self.assertNotIn("has_children", items[0])
        self.assertEqual(items[1]["title"], "filled")
        self.assertNotIn("has_children", items[1])

    def test_has_children_not_returned_in_backend_serialization(self):
        empty = api.content.create(
            container=self.document, type="Document", title="empty"
        )
        filled = api.content.create(
            container=self.document, type="Document", title="filled"
        )

        api.content.create(
            container=self.document["filled"], type="Document", title="Example"
        )
        commit()

        empty_serialization = getMultiAdapter(
            (empty, self.request), ISerializeToJsonSummary
        )()
        filled_serialization = getMultiAdapter(
            (filled, self.request), ISerializeToJsonSummary
        )()

        self.assertNotIn("has_children", empty_serialization)
        self.assertNotIn("has_children", filled_serialization)

    def test_get_content_return_id_value(self):
        news = api.content.create(container=self.portal, type="News Item", title="news")
        commit()

        response = self.api_session.get(news.absolute_url())

        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 2)
        self.assertEqual(items[0]["id"], "multimedia")
        self.assertEqual(items[1]["id"], "documenti-allegati")

    def test_backend_serialization_return_id_value(self):
        serialization = getMultiAdapter(
            (self.document, self.request), ISerializeToJsonSummary
        )()

        self.assertEqual(serialization["id"], "document")

    def test_summary_return_formatted_remote_url_for_links(self):
        link = api.content.create(
            container=self.portal,
            type="Link",
            title="link",
            remoteUrl="	/plone/resolveuid/{}".format(self.document.UID()),
        )
        commit()

        response = self.api_session.get("@search?portal_type=Link")
        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 1)
        self.assertEqual(items[0]["remoteUrl"], self.document.absolute_url())

        serializer = getMultiAdapter((link, self.request), ISerializeToJsonSummary)()
        self.assertEqual(serializer["remoteUrl"], self.document.absolute_url())

    # il campo `ruolo` non è più presente nel tipo Persona
    # def test_summary_return_persona_role(self):
    #     api.content.create(
    #         container=self.portal, type="Persona", title="John Doe", ruolo="unknown"
    #     )
    #     api.content.create(container=self.portal, type="Persona", title="Mario Rossi")

    #     commit()

    #     brains = api.content.find(portal_type="Persona", id="mario-rossi")
    #     results = getMultiAdapter((brains, self.request), ISerializeToJson)(
    #         fullobjects=False
    #     )

    #     self.assertEqual(results["items"][0]["ruolo"], None)
    #     self.assertEqual(results["items"][0]["title"], "Mario Rossi")

    #     brains = api.content.find(portal_type="Persona", id="john-doe")
    #     results = getMultiAdapter((brains, self.request), ISerializeToJson)(
    #         fullobjects=False
    #     )

    #     self.assertEqual(results["items"][0]["ruolo"], "unknown")
    #     self.assertEqual(results["items"][0]["title"], "John Doe")

    #     # test also with restapi call
    #     response = self.api_session.get(
    #         "{}/@search?portal_type=Persona&id=mario-rossi".format(self.portal_url)
    #     )

    #     result = response.json()
    #     items = result.get("items", [])

    #     self.assertEqual(items[0]["title"], "Mario Rossi")
    #     self.assertEqual(items[0]["ruolo"], None)

    #     response = self.api_session.get(
    #         "{}/@search?portal_type=Persona&id=john-doe".format(self.portal_url)
    #     )

    #     result = response.json()
    #     items = result.get("items", [])

    #     self.assertEqual(items[0]["title"], "John Doe")
    #     self.assertEqual(items[0]["ruolo"], "unknown")

    def test_summary_return_design_meta_type(self):
        api.content.create(
            container=self.portal,
            type="News Item",
            title="News",
            tipologia_notizia="foo",
        )
        commit()

        response = self.api_session.get("@search?portal_type=News Item")
        result = response.json()
        items = result.get("items", [])

        # TODO: non basta che `foo` sia una tipologia di notizia, serve che sia presente
        # anche nella tassonomia delle tipologie di notizia

        # self.assertEqual(len(items), 1)
        # self.assertEqual(items[0]["design_italia_meta_type"], "foo")

        # serializer = getMultiAdapter((news, self.request), ISerializeToJsonSummary)()
        # self.assertEqual(serializer["design_italia_meta_type"], "foo")

        # other contents return their name
        response = self.api_session.get("@search?UID={}".format(self.document.UID()))
        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 1)
        self.assertEqual(items[0]["design_italia_meta_type"], "Page")

        serializer = getMultiAdapter(
            (self.document, self.request), ISerializeToJsonSummary
        )()
        self.assertEqual(serializer["design_italia_meta_type"], "Page")

    def test_summary_return_tassonomia_argomenti_expanded(self):
        intids = getUtility(IIntIds)
        argomento = api.content.create(
            container=self.portal,
            type="Pagina Argomento",
            title="Argomento",
        )
        commit()

        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="News",
            tipologia_notizia="foo",
        )

        news.tassonomia_argomenti = [RelationValue(intids.getId(argomento))]
        news.reindexObject()
        commit()

        response = self.api_session.get(
            "@search?portal_type=News Item&metadata_fields=tassonomia_argomenti"
        )
        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 1)
        self.assertEqual(len(items[0]["tassonomia_argomenti"]), 1)
        self.assertEqual(items[0]["tassonomia_argomenti"][0]["title"], argomento.title)

    def test_summary_do_not_return_tassonomia_argomenti_if_not_in_metadata_fields(self):
        intids = getUtility(IIntIds)
        argomento = api.content.create(
            container=self.portal,
            type="Pagina Argomento",
            title="Argomento",
        )
        commit()

        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="News",
            tipologia_notizia="foo",
        )

        news.tassonomia_argomenti = [RelationValue(intids.getId(argomento))]
        news.reindexObject()
        commit()

        response = self.api_session.get("@search?portal_type=News Item")
        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 1)
        self.assertNotIn("tassonomia_argomenti", items[0])

        serializer = getMultiAdapter((news, self.request), ISerializeToJsonSummary)()
        self.assertNotIn("tassonomia_argomenti", serializer)

        response = self.api_session.get(
            "@search?portal_type=News Item&metadata_fields=tassonomia_argomenti"
        )
        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 1)
        self.assertIn("tassonomia_argomenti", items[0])

        self.request.form["metadata_fields"] = "tassonomia_argomenti"
        serializer = getMultiAdapter((news, self.request), ISerializeToJsonSummary)()
        self.assertIn("tassonomia_argomenti", serializer)

        # works also with _all
        response = self.api_session.get(
            "@search?portal_type=News Item&metadata_fields=_all"
        )
        result = response.json()
        items = result.get("items", [])

        self.assertEqual(len(items), 1)
        self.assertIn("tassonomia_argomenti", items[0])

        self.request.form["metadata_fields"] = "_all"
        serializer = getMultiAdapter((news, self.request), ISerializeToJsonSummary)()
        self.assertIn("tassonomia_argomenti", serializer)



# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName

try:
    from plone.base.utils import safe_text
except ImportError:
    from Products.CMFPlone.utils import safe_unicode as safe_text
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
from zope.site.hooks import getSite


class ReferencesVocabulary(object):
    INDEX = ""

    def get_all_index_values(self):
        index = self.catalog._catalog.getIndex(self.INDEX)
        return list(index.uniqueValues())

    def __call__(self, registry=None):
        site = getSite()
        self.catalog = getToolByName(site, "portal_catalog", None)
        if self.catalog is None:
            return SimpleVocabulary([])
        values = self.get_all_index_values()
        brains = self.catalog(UID=values)
        terms = []
        for brain in brains:
            terms.append(SimpleTerm(brain.UID, brain.UID, safe_text(brain.Title)))
        return SimpleVocabulary(terms)


@implementer(IVocabularyFactory)
class EventLocationVocabulary(ReferencesVocabulary):
    INDEX = "event_location"


@implementer(IVocabularyFactory)
class OfficeLocationVocabulary(ReferencesVocabulary):
    INDEX = "ufficio_responsabile"


@implementer(IVocabularyFactory)
class UOLocationVocabulary(ReferencesVocabulary):
    INDEX = "uo_location"


EventLocationVocabularyFactory = EventLocationVocabulary()
OfficeLocationVocabularyFactory = OfficeLocationVocabulary()
UOLocationVocabularyFactory = UOLocationVocabulary()



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from plone.indexer.decorator import indexer


@indexer(IUnitaOrganizzativa)
def uo_location(context, **kw):
    luoghi = []
    for field in ["sede", "sedi_secondarie"]:
        for ref in getattr(context, field, []):
            sede = ref.to_object
            if sede:
                luoghi.append(sede.UID())
    return luoghi



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation import Geolocation
from plone.namedfile.file import NamedBlobImage
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import RelativeSession
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import os
import unittest


class UOSummarySerializerTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def test_geolocation_injected(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO",
            geolocation=Geolocation(25, 25),
        )

        summary = getMultiAdapter(
            (uo, self.layer["request"]), ISerializeToJsonSummary
        )()

        self.assertDictEqual(
            {
                "longitude": uo.geolocation.longitude,
                "latitude": uo.geolocation.latitude,
            },
            summary["geolocation"],
        )

    def test_pdc_and_location(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO",
        )
        uo2 = api.content.create(
            container=uo,
            type="UnitaOrganizzativa",
            title="UO",
        )
        pdc = api.content.create(
            container=self.portal,
            type="PuntoDiContatto",
            title="PDC",
        )
        venue = api.content.create(
            container=self.portal,
            type="Venue",
            title="Venue",
        )
        commit()
        intids = getUtility(IIntIds)
        uo2.sede = [RelationValue(intids.getId(venue))]
        uo2.contact_info = [RelationValue(intids.getId(pdc))]
        commit()
        response = self.api_session.get(uo.absolute_url())
        res = response.json()
        contact_info = res["uo_children"][0]["contact_info"]
        sede = res["uo_children"][0]["sede"]
        self.assertEqual(contact_info[0]["@type"], "PuntoDiContatto")
        self.assertEqual(sede[0]["@type"], "Venue")

    def test_image_in_uo_serializer(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO",
        )
        uo2 = api.content.create(
            container=uo,
            type="UnitaOrganizzativa",
            title="UO",
        )
        commit()
        response = self.api_session.get(uo.absolute_url())
        res = response.json()
        uo_children = res["uo_children"][0]
        self.assertEqual(uo_children["image_field"], None)
        self.assertEqual(uo_children["image_scales"], None)

        # now, add a preview image
        filename = os.path.join(os.path.dirname(__file__), "example.png")
        uo2.preview_image = NamedBlobImage(
            data=open(filename, "rb").read(),
            filename="example.png",
            contentType="image/png",
        )
        commit()
        response = self.api_session.get(uo.absolute_url())
        res = response.json()
        uo_children = res["uo_children"][0]
        self.assertEqual(uo_children["image_field"], "preview_image")
        self.assertEqual("preview_image" in res["uo_children"][0]["image_scales"], True)

        # let's try with both image
        uo2.image = NamedBlobImage(
            data=open(filename, "rb").read(),
            filename="example.png",
            contentType="image/png",
        )
        commit()
        response = self.api_session.get(uo.absolute_url())
        res = response.json()
        image_scales = list(res["uo_children"][0]["image_scales"].keys())
        self.assertIn("preview_image", image_scales)
        self.assertIn("image", image_scales)



from plone import api
from plone.restapi.behaviors import IBlocks
from plone.restapi.indexers import SearchableText_blocks
from Products.Five import BrowserView
from zope.interface import implementer


FLAG = '<i class="fa-solid fa-check"></i>'


def text_in_block(blocks):
    @implementer(IBlocks)
    class FakeObject(object):
        """
        We use a fake object to use SearchableText Indexer
        """

        def Subject(self):
            return ""

        def __init__(self, blocks, blocks_layout):
            self.blocks = blocks
            self.blocks_layout = blocks_layout
            self.id = ""
            self.title = ""
            self.description = ""

    if not blocks:
        return None
    fakeObj = FakeObject(blocks.get("blocks", ""), blocks.get("blocks_layout", ""))
    return SearchableText_blocks(fakeObj)()


class CheckServizi(BrowserView):
    cds = None

    def is_anonymous(self):
        return api.user.is_anonymous()

    def get_canale_accesso_info(self, servizio):
        canale_fisico = getattr(servizio, "canale_fisico", None)
        canale_digitale = text_in_block(getattr(servizio, "canale_digitale", None))
        canale_digitale_link = getattr(servizio, "canale_digitale_link", None)
        if canale_digitale and canale_digitale_link and canale_fisico:
            return "D e F"
        elif canale_digitale and canale_digitale_link:
            return "D"
        elif canale_fisico:
            return "F"
        # elif canale_digitale or canale_digitale_link:
        #     return "&frac12;D"
        return None

    def information_dict(self, servizio):
        return {
            "title": getattr(servizio, "title"),
            "description": getattr(servizio, "description", None),
            "condizioni_di_servizio": getattr(servizio, "condizioni_di_servizio", None),
            "tassonomia_argomenti": getattr(servizio, "tassonomia_argomenti", None),
            "a_chi_si_rivolge": text_in_block(
                getattr(servizio, "a_chi_si_rivolge", None)
            ),
            "come_si_fa": text_in_block(getattr(servizio, "come_si_fa", None)),
            "cosa_si_ottiene": text_in_block(
                getattr(servizio, "cosa_si_ottiene", None)
            ),
            "canale_accesso": self.get_canale_accesso_info(servizio),
            "cosa_serve": text_in_block(getattr(servizio, "cosa_serve", None)),
            "tempi_e_scadenze": text_in_block(
                getattr(servizio, "tempi_e_scadenze", None)
            ),
            "ufficio_responsabile": getattr(servizio, "ufficio_responsabile", None),
        }

    def plone2volto(self, url):
        portal_url = api.portal.get().absolute_url()
        frontend_domain = api.portal.get_registry_record(
            "volto.frontend_domain", default=""
        )
        if frontend_domain and url.startswith(portal_url):
            return url.replace(portal_url, frontend_domain, 1)
        return url

    def get_servizi(self):
        if self.is_anonymous():
            return []
        pc = api.portal.get_tool("portal_catalog")
        brains = pc(portal_type="Servizio")
        results = {}
        for brain in brains:
            servizio = brain.getObject()
            # if safe_hasattr(servizio, "condizioni_di_servizio") and getattr(
            #     servizio, "condizioni_di_servizio"
            # ):
            #     continue

            # Se chiediamo di vedere anche le condizioni di servizio, lo teniamo
            self.cds = self.request.get("condizioni_di_servizio", None)
            information_dict = self.information_dict(servizio)
            if not self.cds:
                del information_dict["condizioni_di_servizio"]

            if all(information_dict.values()):
                continue

            parent = servizio.aq_inner.aq_parent
            if parent.title not in results:
                results[parent.title] = {
                    "url": self.plone2volto(parent.absolute_url()),
                    "children": [],
                }

            results[parent.title]["children"].append(
                {
                    "title": servizio.title,
                    "url": self.plone2volto(servizio.absolute_url()),
                    "data": {
                        "title": information_dict.get("title") and FLAG or "",
                        "description": information_dict.get("description")
                        and FLAG
                        or "",
                        "condizioni_di_servizio": information_dict.get(
                            "condizioni_di_servizio"
                        )
                        and FLAG
                        or "",
                        "tassonomia_argomenti": information_dict.get(
                            "tassonomia_argomenti"
                        )
                        and FLAG
                        or "",
                        "a_chi_si_rivolge": information_dict.get("a_chi_si_rivolge")
                        and FLAG
                        or "",
                        "come_si_fa": information_dict.get("come_si_fa") and FLAG or "",
                        "cosa_si_ottiene": information_dict.get("cosa_si_ottiene")
                        and FLAG
                        or "",
                        "canale_accesso": information_dict.get("canale_accesso") or "",
                        "cosa_serve": information_dict.get("cosa_serve") and FLAG or "",
                        "tempi_e_scadenze": information_dict.get("tempi_e_scadenze")
                        and FLAG
                        or "",
                        "ufficio_responsabile": information_dict.get(
                            "ufficio_responsabile"
                        )
                        and FLAG
                        or "",
                    },
                }
            )

        results = dict(sorted(results.items()))
        for key in results:
            results[key]["children"].sort(key=lambda x: x["title"])
        return results



# -*- coding: utf-8 -*-
from collective.z3cform.datagridfield.interfaces import IRow
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from plone.restapi.types.adapters import ObjectJsonSchemaProvider
from redturtle.volto.types.adapters import (
    TextLineJsonSchemaProvider as BaseTextLineJsonSchemaProvider,
)
from plone.restapi.types.interfaces import IJsonSchemaProvider
from plone.restapi.types.utils import get_fieldsets
from plone.restapi.types.utils import get_jsonschema_properties
from plone.restapi.types.utils import iter_fields
from zope.component import adapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IField
from zope.schema.interfaces import ITextLine
from zope.schema.interfaces import IVocabularyFactory

DATAGRID_FIELDS = ["value_punto_contatto", "timeline_tempi_scadenze"]


@adapter(IField, Interface, Interface)
@implementer(IJsonSchemaProvider)
class LeadImageJsonSchemaProvider(ObjectJsonSchemaProvider):
    def get_size_vocabulary(self):
        factory = getUtility(
            IVocabularyFactory, "design.plone.vocabularies.leadimage_dimension"
        )

        vocabulary = factory(self.context)._terms
        return {term.token: term.title for term in vocabulary if term.token}

    def get_schema(self):
        schema = super(LeadImageJsonSchemaProvider, self).get_schema()
        sizes = self.get_size_vocabulary()
        portal_type = getattr(self.request, "steps")[-1]
        portal_type_size = sizes.get(portal_type, None)
        if portal_type_size:
            msgid = _(
                "image_size_help",
                default="La dimensione dell'immagine dovrebbe essere di ${size} px",  # noqa
                mapping={"size": portal_type_size},
            )
            schema["description"] = translate(msgid, context=self.request)

        return schema


@adapter(IRow, Interface, Interface)
@implementer(IJsonSchemaProvider)
class DataGridRowJsonSchemaProvider(ObjectJsonSchemaProvider):
    def __init__(self, field, context, request):
        super().__init__(field, context, request)
        self.fieldsets = get_fieldsets(context, request, self.field.schema)

    def get_factory(self):
        return "DataGridField Row"

    def get_properties(self):
        if self.prefix:
            prefix = ".".join([self.prefix, self.field.__name__])
        else:
            prefix = self.field.__name__
        return get_jsonschema_properties(
            self.context, self.request, self.fieldsets, prefix
        )

    def additional(self):
        info = super().additional()
        properties = self.get_properties()
        required = []
        for field in iter_fields(self.fieldsets):
            name = field.field.getName()

            # Determine required fields
            if field.field.required:
                required.append(name)

            # Include field modes
            if field.mode:
                properties[name]["mode"] = field.mode

        info["fieldsets"] = [
            {
                "id": "default",
                "title": "Default",
                "fields": [x for x in properties.keys()],
            },
        ]
        info["required"] = required
        info["properties"] = properties
        return info


@adapter(ITextLine, Interface, IDesignPloneContenttypesLayer)
@implementer(IJsonSchemaProvider)
class TextLineJsonSchemaProvider(BaseTextLineJsonSchemaProvider):
    def get_widget(self):
        """
        Force url widget to some fields
        """
        if self.field.__name__ == "canale_digitale_link":
            return "url"
        return super().get_widget()



# -*- coding: utf-8 -*-
from AccessControl.unauthorized import Unauthorized
from Acquisition import aq_inner
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from design.plone.contenttypes.interfaces.servizio import IServizio
from plone import api
from plone.app.contenttypes.utils import replace_link_variables_by_paths
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile.interfaces import INamedFileField
from plone.outputfilters.browser.resolveuid import uuidToURL
from plone.restapi.interfaces import IBlockFieldSerializationTransformer
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.serializer.dxfields import DefaultFieldSerializer
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import subscribers
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IList
from zope.schema.interfaces import ISourceText
from zope.schema.interfaces import ITextLine

import json
import re

RESOLVEUID_RE = re.compile(".*?/resolve[Uu]id/([^/]*)/?(.*)$")
KEYS_WITH_URL = ["linkUrl", "navigationRoot", "showMoreLink"]


@implementer(IFieldSerializer)
@adapter(ISourceText, IDexterityContent, IDesignPloneContenttypesLayer)
class SourceTextSerializer(DefaultFieldSerializer):
    def __call__(self):
        value = super(SourceTextSerializer, self).__call__()
        if self.field.getName() == "search_sections":
            return json.dumps(serialize_data(context=self.context, json_data=value))
        return value


@implementer(IFieldSerializer)
@adapter(IList, IServizio, IDesignPloneContenttypesLayer)
class TempiEScadenzeValueSerializer(DefaultFieldSerializer):
    def __call__(self):
        value = super(TempiEScadenzeValueSerializer, self).__call__()

        patched_timeline = []
        if self.field.getName() == "timeline_tempi_scadenze" and value:
            for entry in value:
                patched_timeline.append(entry)
            return json_compatible(patched_timeline)
        return value


@adapter(INamedFileField, IDexterityContent, IDesignPloneContenttypesLayer)
class FileFieldViewModeSerializer(DefaultFieldSerializer):
    """Ovveride the basic DX serializer to handle the visualize file functionality"""

    def __call__(self):
        namedfile = self.field.get(self.context)
        if namedfile is None:
            return

        url = "/".join(
            (
                self.context.absolute_url(),
                self.get_file_view_mode(namedfile.contentType),
                self.field.__name__,
            )
        )
        result = {
            "filename": namedfile.filename,
            "content-type": namedfile.contentType,
            "size": namedfile.getSize(),
            "download": url,
        }

        return json_compatible(result)

    def get_file_view_mode(self, content_type):
        """Pdf view depends on the visualize_files property in thq aq_chain"""
        if self.context and "pdf" in content_type:
            if getattr(aq_inner(self.context), "visualize_files", None):
                return "@@display-file"

        return "@@download"


def serialize_data(context, json_data, show_children=False):
    request = getRequest()
    if not json_data:
        return ""
    data = json.loads(json_data)
    for root in data:
        for tab in root.get("items", []):
            for key in KEYS_WITH_URL:
                value = tab.get(key, [])
                if value:
                    serialized = []
                    for uid in value:
                        try:
                            item = api.content.get(UID=uid)
                        except Unauthorized:
                            # private item and user can't see it
                            continue
                        if not item:
                            continue
                        summary = getMultiAdapter(
                            (item, request), ISerializeToJsonSummary
                        )()
                        if summary:
                            # serializer doesn't return uid
                            summary["UID"] = uid
                            if show_children:
                                summary["items"] = get_item_children(item)
                            serialized.append(summary)
                    tab[key] = serialized
            fix_blocks(context=context, tab=tab)
    return json_compatible(data)


def fix_blocks(context, tab):
    request = getRequest()
    blocks = tab.get("blocks", {})
    if blocks:
        for id, block_value in blocks.items():
            block_type = block_value.get("@type", "")
            handlers = []
            for h in subscribers(
                (context, request),
                IBlockFieldSerializationTransformer,
            ):
                if h.block_type == block_type or h.block_type is None:
                    handlers.append(h)
            for handler in sorted(handlers, key=lambda h: h.order):
                block_value = handler(block_value)

            blocks[id] = block_value


def get_item_children(item):
    path = "/".join(item.getPhysicalPath())
    query = {
        "path": {"depth": 1, "query": path},
        "sort_on": "getObjPositionInParent",
        "exclude_from_nav": False,
    }
    brains = api.content.find(**query)
    return [
        getMultiAdapter((brain, getRequest()), ISerializeToJsonSummary)()
        for brain in brains
    ]


@adapter(ITextLine, IServizio, IDesignPloneContenttypesLayer)
class ServizioTextLineFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        value = self.get_value()
        if self.field.getName() != "canale_digitale_link" or not value:
            return super().__call__()

        path = replace_link_variables_by_paths(context=self.context, url=value)
        match = RESOLVEUID_RE.match(path)
        if match:
            uid, suffix = match.groups()
            value = uuidToURL(uid)
        else:
            portal = getMultiAdapter(
                (self.context, self.context.REQUEST), name="plone_portal_state"
            ).portal()
            ref_obj = portal.restrictedTraverse(path, None)
            if ref_obj:
                value = ref_obj.absolute_url()

        return json_compatible(value)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from design.plone.contenttypes.interfaces.modulo import IModulo
from design.plone.contenttypes.restapi.serializers.summary import (
    DefaultJSONSummarySerializer,
)
from plone.dexterity.utils import iterSchemata
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJsonSummary
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.schema import getFields


@implementer(ISerializeToJsonSummary)
@adapter(IModulo, IDesignPloneContenttypesLayer)
class SerializeModuloToJsonSummary(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        summary = super().__call__(force_all_metadata=force_all_metadata)
        fields = [
            "file_principale",
            "formato_alternativo_1",
            "formato_alternativo_2",
        ]
        for schema in iterSchemata(self.context):
            for name, field in getFields(schema).items():
                if name not in fields:
                    continue

                # serialize the field
                serializer = queryMultiAdapter(
                    (field, self.context, self.request), IFieldSerializer
                )
                value = serializer()
                summary[name] = value
        return summary



# -*- coding: utf-8 -*-
from collective.geolocationbehavior.geolocation import IGeolocatable
from collective.venue.interfaces import IVenue
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IGeolocatableUnitaOrganizzativa(IGeolocatable):
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["geolocation"],
    )


@provider(IFormFieldProvider)
class IGeolocatableVenue(IGeolocatable):
    model.fieldset(
        "dove", label=_("dove_label", default="Dove"), fields=["geolocation"]
    )


@provider(IFormFieldProvider)
class IGeolocatableEvent(IGeolocatable):
    model.fieldset(
        "luogo",
        label=_("luogo_label", default="Luogo"),
        fields=["geolocation"],
    )


@implementer(IGeolocatableUnitaOrganizzativa)
@adapter(IUnitaOrganizzativa)
class GeolocatableUnitaOrganizzativa(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IGeolocatableVenue)
@adapter(IVenue)
class GeolocatableVenue(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IGeolocatableEvent)
@adapter(IDexterityContent)
class GeolocatableEvent(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from .related_news_serializer import SerializeFolderToJson as RelatedNewsSerializer
from Acquisition import aq_inner
from collective.venue.interfaces import IVenue
from design.plone.contenttypes.restapi.serializers.summary import (
    DefaultJSONSummarySerializer,
)
from design.plone.contenttypes.restapi.serializers.summary import (
    get_taxonomy_information,
)
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from zc.relation.interfaces import ICatalog
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.security import checkPermission


@implementer(ISerializeToJson)
@adapter(IVenue, Interface)
class VenueSerializer(RelatedNewsSerializer):
    def search(self, index, UID):
        catalog = api.portal.get_tool("portal_catalog")
        query = {
            index: UID,
            "portal_type": ["Servizio"],
            "sort_on": "sortable_title",
            "sort_order": "ascending",
        }

        return catalog(**query)

    def get_venue_services(self, result):
        brains = self.search("service_venue", result["UID"])
        servizi = [
            getMultiAdapter((x, self.request), ISerializeToJsonSummary)()
            for x in brains
        ]
        return servizi

    def get_venue_offices(self, result):
        catalog = getUtility(ICatalog)
        intids = getUtility(IIntIds)
        offices = []
        for attr in ["sede", "sedi_secondarie"]:
            relations = catalog.findRelations(
                dict(
                    to_id=intids.getId(aq_inner(self.context)),
                    from_attribute=attr,
                )
            )

            for rel in relations:
                obj = intids.queryObject(rel.from_id)
                if obj is not None and checkPermission("zope2.View", obj):
                    summary = getMultiAdapter(
                        (obj, getRequest()), ISerializeToJsonSummary
                    )()
                    offices.append(summary)
        return sorted(offices, key=lambda k: k["title"])

    def __call__(self, version=None, include_items=True):
        self.index = "news_venue"
        result = super(VenueSerializer, self).__call__(version=None, include_items=True)
        result["venue_services"] = self.get_venue_services(result)
        result["sede_di"] = self.get_venue_offices(result)
        return result


@implementer(ISerializeToJsonSummary)
@adapter(IVenue, Interface)
class SerializeVenueToJsonSummary(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        summary = super().__call__(force_all_metadata=force_all_metadata)
        fields = [
            "street",
            "zip_code",
            "city",
            "country",
            "geolocation",
            "orario_pubblico",
            "telefono",
            "fax",
            "email",
            "pec",
            "web",
            "riferimento_telefonico_struttura",
            "riferimento_mail_struttura",
            "riferimento_pec_struttura",
            "quartiere",
            "nome_sede",
            "title",
            "circoscrizione",
        ]

        for field in fields:
            value = getattr(self.context, field, None)
            if callable(value):
                value = value()
            summary[field] = json_compatible(value)

        get_taxonomy_information("tipologia_luogo", self.context, summary)

        return summary



# -*- coding: utf-8 -*-
"""
We need a solution like that because for some different reasons:
 * we have to customize both SerializeToJson and SerializeFolderToJson
 * If we override SerializeToJson adding this base design_italia_meta_type
   information, we need to override SerializeFolderToJson copying the whole
   code otherwise it will use the code from original SerializeToJson due to
   inheritance
 * Using a monkey patch is the easiest way to include future changes on base
   SerializeToJson and SerializeFolderToJson classes
"""

from collective.taxonomy import PATH_SEPARATOR
from collective.taxonomy.interfaces import ITaxonomy
from plone import api
from plone.restapi.batching import HypermediaBatch
from plone.restapi.deserializer import boolean_value
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.dxcontent import SerializeFolderToJson
from plone.restapi.serializer.dxcontent import SerializeToJson
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate


original_serialize_to_json__call__ = SerializeToJson.__call__


def design_italia_serialize_to_json_call(self, version=None, include_items=True):
    ttool = api.portal.get_tool("portal_types")
    result = original_serialize_to_json__call__(
        self, version=version, include_items=include_items
    )
    result["design_italia_meta_type"] = translate(
        ttool[self.context.portal_type].Title(), context=self.request
    )
    if self.context.portal_type == "News Item":
        if self.context.tipologia_notizia:
            taxonomy = getUtility(
                ITaxonomy, name="collective.taxonomy.tipologia_notizia"
            )
            taxonomy_voc = taxonomy.makeVocabulary(self.request.get("LANGUAGE"))

            title = taxonomy_voc.inv_data.get(self.context.tipologia_notizia, None)

            if title and title.startswith(PATH_SEPARATOR):
                result["design_italia_meta_type"] = title.replace(PATH_SEPARATOR, "", 1)
    return result


def patch_base_serializer():
    SerializeToJson.__call__ = design_italia_serialize_to_json_call


def design_italia_serialize_folder_to_json_call(self, version=None, include_items=True):
    folder_metadata = super(SerializeFolderToJson, self).__call__(
        version=version, include_items=include_items
    )

    folder_metadata.update({"is_folderish": True})
    result = folder_metadata
    include_items = self.request.form.get("include_items", include_items)
    include_items = boolean_value(include_items)
    if include_items:
        query = self._build_query()

        catalog = getToolByName(self.context, "portal_catalog")
        brains = catalog(query)

        batch = HypermediaBatch(self.request, brains)

        # These lines generate wrong result in field @id of the returned items
        # if "fullobjects" not in self.request.form:
        #   result["@id"] = batch.canonical_url
        result["items_total"] = batch.items_total
        if batch.links:
            result["batching"] = batch.links
        if "fullobjects" in list(self.request.form):
            result["items"] = getMultiAdapter((brains, self.request), ISerializeToJson)(
                fullobjects=True
            )["items"]
        else:
            result["items"] = [
                getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
                for brain in batch
            ]
    return result


def patch_base_folder_serializer():
    SerializeFolderToJson.__call__ = design_italia_serialize_folder_to_json_call



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from design.plone.contenttypes.controlpanels.geolocation_defaults import (
    IGeolocationDefaults,
)
from plone import api
from plone.restapi.services.types.get import TypesGet as BaseGet
from zope.i18n import translate
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse


class FieldsetsMismatchError(Exception):
    """Exception thrown when we try to reorder fieldsets, but the order list is
    different from the fieldsets returned from Plone
    """


FIELDSETS_ORDER = {
    "Document": [
        "default",
        "testata",
        "settings",
        "correlati",
        "categorization",
        "dates",
        "ownership",
        "layout",
    ],
    "Documento": [
        "default",
        "descrizione",
        "informazioni",
        "settings",
        "correlati",
        "categorization",
        "dates",
        "ownership",
    ],
    "Event": [
        "default",
        "cose",
        "luogo",
        "date_e_orari",
        "costi",
        "contatti",
        "informazioni",
        "correlati",
        "categorization",
        "dates",
        "settings",
        "ownership",
    ],
    "Incarico": [
        "default",
        "informazioni_compensi",
        "date_e_informazioni",
    ],
    "News Item": [
        "default",
        "dates",
        "correlati",
        "categorization",
        "settings",
        "ownership",
    ],
    "Modulo": [
        "default",
        "settings",
        "correlati",
        "categorization",
        "dates",
        "ownership",
    ],
    "Pagina Argomento": [
        "default",
        "informazioni",
        "correlati",
        "categorization",
        "dates",
        "settings",
        "layout",
        "ownership",
    ],
    "Persona": [
        "default",
        "ruolo",
        "contatti",
        "documenti",
        "informazioni",
        "correlati",
        "categorization",
        "dates",
        "ownership",
        "settings",
    ],
    "PuntoDiContatto": [
        "default",
    ],
    "Servizio": [
        "default",
        "cose",
        "a_chi_si_rivolge",
        "accedi_al_servizio",
        "cosa_serve",
        "costi_e_vincoli",
        "tempi_e_scadenze",
        "casi_particolari",
        "contatti",
        "documenti",
        "link_utili",
        "informazioni",
        "correlati",
        "categorization",
        "settings",
        "ownership",
        "dates",
    ],
    "UnitaOrganizzativa": [
        "default",
        "cosa_fa",
        "struttura",
        "persone",
        "contatti",
        "correlati",
        "categorization",
        "informazioni",
        "settings",
        "ownership",
        "dates",
    ],
    "Venue": [
        "default",
        "descrizione",
        "accesso",
        "dove",
        "orari",
        "contatti",
        "informazioni",
        "settings",
        "correlati",
        "categorization",
    ],
}


@implementer(IPublishTraverse)
class TypesGet(BaseGet):
    def customize_document_schema(self, result):
        moved = ["image", "image_caption", "preview_image"]
        removed = ["preview_caption"]
        for fieldset in result.get("fieldsets", []):
            if fieldset.get("id", "") == "testata":
                fieldset["fields"] = moved + fieldset["fields"]
            if fieldset.get("id", "") == "default":
                fieldset["fields"] = [
                    x for x in fieldset["fields"] if x not in moved and x not in removed
                ]
        return result

    def customize_persona_schema(self, result):
        if "title" in result["properties"]:
            msgid = _("Nome e Cognome", default="Nome e cognome")
            result["properties"]["title"]["title"] = translate(
                msgid, context=self.request
            )
        return result

    def customize_venue_schema(self, result):
        """
        Unico modo per spostare il campo "notes"
        """
        result.get("required").append("description")
        result.get("required").append("image")
        result.get("required").append("street")
        result.get("required").append("city")
        result.get("required").append("zip_code")
        result.get("required").append("geolocation")

        if "properties" in result:
            if "country" in result["properties"]:
                if not result["properties"]["country"].get("default", ""):
                    result["properties"]["country"]["default"] = {
                        "title": "Italia",
                        "token": "380",
                    }
            if "city" in result["properties"]:
                if not result["properties"]["city"].get("default", ""):
                    result["properties"]["city"][
                        "default"
                    ] = api.portal.get_registry_record(
                        "city", interface=IGeolocationDefaults
                    )
            if "zip_code" in result["properties"]:
                if not result["properties"]["zip_code"].get("default", ""):
                    result["properties"]["zip_code"][
                        "default"
                    ] = api.portal.get_registry_record(
                        "zip_code", interface=IGeolocationDefaults
                    )

            if "street" in result["properties"]:
                if not result["properties"]["street"].get("default", ""):
                    result["properties"]["street"][
                        "default"
                    ] = api.portal.get_registry_record(
                        "street", interface=IGeolocationDefaults
                    )

            if "geolocation" in result["properties"]:
                if not result["properties"]["geolocation"].get("default", {}):
                    result["properties"]["geolocation"]["default"] = eval(
                        api.portal.get_registry_record(
                            "geolocation", interface=IGeolocationDefaults
                        )
                    )

        for fieldset in result["fieldsets"]:
            if fieldset.get("id") == "default" and "notes" in fieldset["fields"]:
                fieldset["fields"].remove("notes")
            if fieldset.get("id") == "dove" and "notes" not in fieldset["fields"]:
                fieldset["fields"].append("notes")

        return result

    def customize_versioning_fields_fieldset(self, result):
        """
        Unico modo per spostare i campi del versioning.
        Perché changeNotes ha l'order after="*" che vince su tutto.
        """
        versioning_fields = ["versioning_enabled", "changeNote"]
        for field in versioning_fields:
            found = False
            for fieldset in result["fieldsets"]:
                if fieldset.get("id") == "default" and field in fieldset["fields"]:
                    found = True
                    fieldset["fields"].remove(field)
                if fieldset.get("id") == "settings" and found:
                    fieldset["fields"].append(field)

        return result

    def customize_servizio_schema(self, result):
        result.get("required").append("description")
        return result

    def customize_evento_schema(self, result):
        result.get("required").append("description")
        return result

    def customize_uo_schema(self, result):
        result.get("required").append("description")
        versioning_fields = ["contact_info"]
        for field in versioning_fields:
            for fieldset in result["fieldsets"]:
                if fieldset.get("id") == "contatti" and field in fieldset["fields"]:
                    fieldset["fields"].remove(field)
                    fieldset["fields"].insert(0, field)
        return result

    def customize_news_schema(self, result):
        result.get("required").append("description")
        if self.context.portal_type == "News Item":
            # we are in a news and not in container
            review_state = self.context.portal_workflow.getInfoFor(
                self.context, "review_state"
            )
            if review_state == "published":
                result.get("required").append("effective")

        return result

    def customize_documento_schema(self, result):
        result.get("required").append("description")
        return result

    def reply(self):
        result = super(TypesGet, self).reply()

        if "fieldsets" in result:
            result["fieldsets"] = self.reorder_fieldsets(original=result["fieldsets"])
        pt = self.request.PATH_INFO.split("/")[-1]

        # be careful: result could be dict or list. If list it will not
        # contains title. And this is ok for us.
        pt = self.request.PATH_INFO.split("/")[-1]

        if "title" in result:
            if pt == "Persona":
                result = self.customize_persona_schema(result)
            if pt == "Venue":
                result = self.customize_venue_schema(result)
            if pt == "Document":
                result = self.customize_document_schema(result)
            if pt == "Servizio":
                result = self.customize_servizio_schema(result)
            if pt == "UnitaOrganizzativa":
                result = self.customize_uo_schema(result)
            if pt == "News Item":
                result = self.customize_news_schema(result)
            if pt == "Documento":
                result = self.customize_documento_schema(result)
            if pt == "Event":
                result = self.customize_evento_schema(result)
            result = self.customize_versioning_fields_fieldset(result)
        return result

    def get_order_by_type(self, portal_type):
        return [x for x in FIELDSETS_ORDER.get(portal_type, [])]

    def reorder_fieldsets(self, original):
        pt = self.request.PATH_INFO.split("/")[-1]
        order = self.get_order_by_type(portal_type=pt)
        if not order:
            # no match
            return original
        original_fieldsets = [x["id"] for x in original]

        for fieldset_id in original_fieldsets:
            # if some fieldsets comes from additional addons (not from the
            # base ones), then append them to the order list.
            if fieldset_id not in order:
                order.append(fieldset_id)

        # create a new fieldsets list with the custom order
        new = []
        for id in order:
            for fieldset in original:
                if fieldset["id"] == id:
                    new.append(fieldset)
        if not new:
            # no match
            return original
        return new



# -*- coding: utf-8 -*-
from plone.dexterity.content import Container
from plone.event.interfaces import IEvent
from zope.interface import implementer


@implementer(IEvent)
class Event(Container):
    """ """



# -*- coding: utf-8 -*-
from .related_news_serializer import SerializeFolderToJson
from Acquisition import aq_inner
from design.plone.contenttypes.interfaces.punto_di_contatto import IPuntoDiContatto
from design.plone.contenttypes.restapi.serializers.summary import (
    DefaultJSONSummarySerializer,
)
from plone.dexterity.utils import iterSchemata
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from zc.relation.interfaces import ICatalog
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.schema import getFields
from zope.security import checkPermission


@implementer(ISerializeToJsonSummary)
@adapter(IPuntoDiContatto, Interface)
class SerializePuntoDiContattoToJsonSummary(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        summary = super().__call__(force_all_metadata=force_all_metadata)
        fields = ["value_punto_contatto"]
        for schema in iterSchemata(self.context):
            for name, field in getFields(schema).items():
                if name not in fields:
                    continue

                # serialize the field
                serializer = queryMultiAdapter(
                    (field, self.context, self.request), IFieldSerializer
                )
                value = serializer()
                summary[name] = value

        return summary


@implementer(ISerializeToJson)
@adapter(IPuntoDiContatto, Interface)
class PuntoDiContattoSerializer(SerializeFolderToJson):
    index = ""

    def related_contents(self, field, portal_type):
        """ """
        catalog = getUtility(ICatalog)
        intids = getUtility(IIntIds)
        items = []
        relations = catalog.findRelations(
            dict(
                to_id=intids.getId(aq_inner(self.context)),
                from_attribute=field,
            )
        )

        for rel in relations:
            obj = intids.queryObject(rel.from_id)
            if (
                obj is not None
                and checkPermission("zope2.View", obj)
                and getattr(obj, "portal_type", None) == portal_type
            ):
                summary = getMultiAdapter(
                    (obj, getRequest()), ISerializeToJsonSummary
                )()
                items.append(summary)
        return sorted(items, key=lambda k: k["title"])

    def __call__(self, version=None, include_items=True):
        result = super(PuntoDiContattoSerializer, self).__call__(
            version=version, include_items=include_items
        )
        strutture_correlate = self.related_contents(
            field="contact_info", portal_type="UnitaOrganizzativa"
        )
        servizi_correlati = self.related_contents(
            field="contact_info", portal_type="Servizio"
        )
        luoghi_correlati = self.related_contents(
            field="contact_info", portal_type="Venue"
        )
        persone_correlate = self.related_contents(
            field="contact_info", portal_type="Persona"
        )

        if strutture_correlate:
            result["strutture_correlate"] = strutture_correlate
        if servizi_correlati:
            result["servizi_correlati"] = servizi_correlati
        if luoghi_correlati:
            result["luoghi_correlati"] = luoghi_correlati
        if persone_correlate:
            result["persone_correlate"] = persone_correlate

        return result



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobFile
from plone.restapi.testing import RelativeSession
from transaction import commit

import unittest


class SummarySerializerTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.cartella_modulistica = api.content.create(
            container=self.portal,
            type="CartellaModulistica",
            title="Cartella Modulistica",
        )
        self.document = api.content.create(
            container=self.cartella_modulistica, type="Documento", title="Document"
        )
        self.modulo = api.content.create(
            container=self.document,
            type="Modulo",
            title="Modulo",
            file_principale=NamedBlobFile("some data", filename="file.pdf"),
        )
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_if_visualize_files_false_so_download(self):
        response = self.api_session.get(self.modulo.absolute_url()).json()

        self.assertIn("@@download", response["file_principale"]["download"])

    def test_if_visualize_files_true_so_dsiplay(self):
        self.cartella_modulistica.visualize_files = True

        commit()

        response = self.api_session.get(self.modulo.absolute_url()).json()

        self.assertIn("@@display-file", response["file_principale"]["download"])



# -*- coding: utf-8 -*-


def onModify(context, event):
    for description in event.descriptions:
        if "IBasic.title" in getattr(description, "attributes", []):
            for child in context.listFolderContents():
                child.reindexObject(idxs=["parent"])
